#include "socket.h"


/**
 * Constructors/Destructors
 */
/**
 * Methods
 */
/**
 * Get the value of family
 * family of this network communication, see socket(3)
 * Families  are defined in <sys/socket.h>:
 * PF_UNIX, PF_LOCAL   Local communication  
 * PF_INET             IPv4 Internet protocols         
 * PF_INET6           IPv6 Internet protocols
 * PF_PACKET        Low level packet interface       packet(7)
 * there are more but these are the moast important 
 * @return the value of family
 */
int Socket::get_family ( ) {
  return family;
}

  
/**
 * Get the value of socktype
 * Type of the socket also see socket(3)
 * SOCK_RAW and SOCK_STREAM are the most important
 * 
 * SOCK_STREAM
 *               Provides sequenced, reliable, two-way, connection-based byte  streams.
 *               An out-of-band data transmission mechanism may be supported.
 * 
 * SOCK_SEQPACKET
 *               Provides a sequenced, reliable, two-way connection-based  data  trans-
 *               mission  path  for  datagrams  of  fixed maximum length; a consumer is
 *               required to read an entire packet with each read system call.
 * SOCK_RAW
 *               Provides raw network protocol access.
 * 
 * 
 * @return the value of socktype
 */
int Socket::get_socktype ( ) {
  return socktype;
}

/**
 * Get the value of protocol
 * Protocol to use. see socket(3) and protocols(5)
 * Just use 0 if you have socktype SOCK_STREAM 
 * @return the value of protocol
 */
int Socket::get_protocol ( ) {
  return protocol;
}

/**
 * Get the value of lastError
 * last error occured in any socket operation
 * @return the value of lastError
 */
SocketException* Socket::get_lastError ( ) {
  return lastError;
}

/**
 * Get the value of timeout
 * time in microsecondss this socket will wait for connection
 * @return the value of timeout
 */
int Socket::get_timeout ( ) {
  return timeout;
}
/**
 * Set the value of timeout
 * time in microsecondss this socket will wait for connection
 * @param value the value of timeout
 */
void Socket::set_timeout (int value ) {
  timeout = value;
}

/**
 * create a new general purpose Socket
 * wrapper to ::socket()
 */
Socket::Socket (int domain, int type, int protocol) {
  //timeout=	16777216;
  timeout =0;
  std::cout << "[ok] domain: " << domain << " TYPE: " << type <<  "  PROTO: " <<protocol << std::endl;
  sock = ::socket(domain, type,protocol);
  if(sock <0){
	throw SocketException("Could not create socket - ","",errno);
  }
  else{
	int reuse=1;
	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,  &reuse, sizeof (reuse) );
	incRef();
  }
}
/**
 * read a line from this socket
 * @return 1 if sucessful 0 otherwise
 */
int Socket::readLine (char* buffer, int length, char term) {
  std::cout<<"[ok] reading \n";
  int readin=0, stat=0,retval=1;
  char buf;
  errno =0;
  if (sock < 0) {
	throw SocketException(" Error reading from Socket","Socket not initialized",errno);
  }
  
  while (readin < length - 2) {
	if (timeout && TimedOut()){
	  throw SocketException("Error reading from Socket","socket timed out",errno);
	}	  
	stat = ::read(sock, &buf, 1);
	if (stat == -1) {
	  retval = 0;
	  throw SocketException(" Error reading from Socket","stat < 0 Socket Error",errno);
	}
	if (stat==0 || buf == term){		
	  retval=0;
	  break;
	}
	buffer[readin]=buf;
	readin++;
	std::cout << buf;
  }

  if (term != '\0'){
	readin++;
  }
  buffer[readin] = '\0';
  return retval;
}

  
/**
 * Write a line on this socket to the other side
 * @return int 1 if sucessful, 0 otherwise
 */
int Socket::writeLine (const char* buffer, char term) {
  int written = 0;
  int len = strlen(buffer);
  if (term == '\0')			// If terminator is '\0', include that in data out
	len++;
  if (sock < 0) {
	throw SocketException("Error writing on Socket ","Socket not initialized",errno);
  }
  while (len > 0) {
	written = write(sock, buffer, len);
	if (written <= 0) {
	  throw SocketException("Error writing on Socket "," see errorcode ",errno);
	  return -1; 
	}
	buffer += written;
	len -= written;
  }
  return 0;
}

/**
 * read message from socket into
 * @param buffer container for read datagrams
 * @param length how much to read in a single call
 * @return 1 if sucessful 0 otherwise
 */
int Socket::readMsg( char* buffer, int length){
  int status = ::recv (sock , buffer, length, 0 );
  if ( status == -1 || status == 0){
	throw SocketException("Error reading from socket using readMsg().","Don't know the cause",errno);
	return 0;
  }
  else {
	return status;
  }
}
/**
 * Write a message on the Socket
 * @param msg message to write;
 * @return 1 if sucessful 0 otherwise
 */
int Socket::sendMsg(string msg){
  int status = ::send (sock, msg.c_str(), msg.size(), MSG_NOSIGNAL );
  if ( status == -1 ) {
	return 1;
  }
  else {
	return 0;
  }
}


/**
 * check if Socket timed out and refresh time to since last access
 * @return int 1 if sucessful, 0 otherwise
 */
int Socket::TimedOut(){
  fd_set fdSet;
  struct timeval TimeToWait;
  TimeToWait.tv_sec  = timeout / 10000;
  TimeToWait.tv_usec = timeout % 10000;
  FD_ZERO(&fdSet);
  FD_SET(sock, &fdSet);
  int Ret = select(sock+1, &fdSet, &fdSet, &fdSet, &TimeToWait);
  if (Ret <= 0){
	throw SocketException("Error selecting Socket for timeout ","Timeout Error",errno);
  }
  return Ret;
}


  
/**
 * Get the value of port
 * TCP port for network traffic
 * @return the value of port
 */
int Socket::get_port ( ) {
  return port;
}

/**
 * Get the value of address
 * C struct to store the relevant information
 * @return the value of sockaddr
 */
sockaddr_in Socket::get_sockaddr ( ) {
  return address;
}


/**
 * Close the socket (terminating all communication)
 * @return 1 if sucessful 0 otherwise
 */
int Socket::close () {
  if(sock <= 0 ){
	throw SocketException("Error closing Socket","Socket already closed or not opened",errno);
  }
  else{
	decRef();
	if(references[sock] == 0){
	  ::close(sock);	
	}
  }
}

/**
 * set the own sock handle to this new one
 */
void Socket::operator = (int newsock) {
  sock = newsock;
}
/**
 * change the own socket handle (sock) to the handle of the other and close the old one
 */
Socket& Socket::operator = (Socket& newsock) {
  if( &newsock != this ) // Make sure caller didn't to X = X;
	*this = newsock.sock;
  return( *this );
}
/**
 * Increment the map by one so we make sure no reference for this socket exists anywhere in the programm when we destry it.
 * @return 1 if sucessful 0 otherwise
 */
int Socket::incRef () {
  if( references.find(sock)!=references.end() ){
	references[sock]++;
  }
  else{
	references[sock]=1;
  }
}

/**
 * decrement the map that indicates all other objects that still have a referenceto this socket
 * @return 1 if sucessful 0 otherwise
 */
int Socket::decRef () {
  if( references.find(sock)!=references.end() ){
	references[sock]--;
  }
}
  

/**
 *  Wrapper for output
 */
Socket & operator<<(Socket & out, const char *cpStr){
  out.writeLine(cpStr);
  return (out);
}


/**
 *  Wrapper for input
 */
Socket & operator>>(Socket & in, char *cpStr){
  in.readLine(cpStr, 0x0FFFF);	// No good number here, really
  return (in);
}
	  
  


