#ifndef SOCKET_H
#define SOCKET_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <cstdlib>
#include <iostream>
#include <string>
#include <map>

#include "socketexception.h"


using std::string;
using std::map;
  /**
   * Class Socket
   * A all purpose socket for any network activity
   * can be used to derive subclasses from.
   * Actually this class is a combination of all nice features from http://www.codefighter.com/linux2.html and Rob Tougher in his article on LinuxGazette
   */
  class Socket {
	/**
	 * Public stuff
	 */
  public:
	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of family
	 * family of this network communication, see socket(3)
	 * Families  are defined in <sys/socket.h>:
	 * PF_UNIX, PF_LOCAL   Local communication
	 * PF_INET             IPv4 Internet protocols
	 * PF_INET6           IPv6 Internet protocols
	 * PF_PACKET        Low level packet interface       packet(7)
	 * there are more but these are the moast important
	 * @return the value of family
	 */
	int get_family ( );


	/**
	 * Get the value of socktype
	 * Type of the socket also see socket(3)
	 * SOCK_RAW and SOCK_STREAM are the most important
	 *
	 * SOCK_STREAM
	 *               Provides sequenced, reliable, two-way, connection-based byte  streams.
	 *               An out-of-band data transmission mechanism may be supported.
	 *
	 * SOCK_SEQPACKET
	 *               Provides a sequenced, reliable, two-way connection-based  data  trans-
	 *               mission  path  for  datagrams  of  fixed maximum length; a consumer is
	 *               required to read an entire packet with each read system call.
	 * SOCK_RAW
	 *               Provides raw network protocol access.
	 *
	 *
	 * @return the value of socktype
	 */
	int get_socktype ( );


	/**
	 * Get the value of protocol
	 * Protocol to use. see socket(3) and protocols(5)
	 * Just use 0 if you have socktype SOCK_STREAM
	 * @return the value of protocol
	 */
	int get_protocol ( );

	/**
	 * Get the value of lastError
	 * last error occured in any socket operation
	 * @return the value of lastError
	 */
	SocketException* get_lastError ( );


	/**
	 * Get the value of timeout
	 * time in microsecondss this socket will wait for connection
	 * @return the value of timeout
	 */
	int get_timeout ( );


	/**
	 * Set the value of timeout
	 * time in microsecondss this socket will wait for connection
	 * @param value the value of timeout
	 */
	void set_timeout (int value );

	/**
	 * Get the value of port
	 * TCP port for network traffic
	 * @return the value of port
	 */
	int get_port ( );


	/**
	 * Set the value of port
	 * TCP port for network traffic
	 * @param value the value of port
	 */
	void set_port (int value );


	/**
	 * Get the value of sockaddr
	 * C struct to store the relevant information
	 * @return the value of sockaddr
	 */
	sockaddr_in get_sockaddr ( );

	/**
	 * Operations
	 */
	/**
	 * create a new general purpose Socket
	 * wrapper to ::socket()
	 */
	Socket (int domain=PF_INET, int type=SOCK_STREAM, int protocol=0);
		
	
	/**
	 * read a line from this socket
	 * @return 1 if sucessful 0 otherwise
	 */
	int  readLine (char* buffer, int length=1024, char term='\n');

	/**
	* read message from socket into
	* @param buffer container for read datagrams
	* @param length how much to read in a single call
	* @param term terminating symbol of message
	* @return 1 if sucessful 0 otherwise
	*/
	int Socket::readMsg( char* buffer, int length);

	
		
	
	/**
	 * Write a line on this socket to the other side
	 * @return 1 if sucessful 0 otherwise
	 */
	int  writeLine (const char* buffer, char term='\n');

	/**
	* Write a message on the Socket
	* @param msg message to write;
	* @return 1 if sucessful 0 otherwise
	*/
	int Socket::sendMsg(string msg);

	/**
	 * Close the socket (terminating all communication)
	 * @return 1 if sucessful 0 otherwise
	 */
	int  close ();
		
	
	/**
	 * change the own socket handle (sock) to the handle of the other and close the old one
	 */
	Socket&  operator = (Socket& sock);
		
	
	/**
	 * set the own sock handle to this new one
	 */
	void  operator = (int newsock);

	/**
	 * check if Socket timed out and refresh time to since last access
	 * @return int 1 if sucessful, 0 otherwise
	 */
	int TimedOut();
		
	
	/**
	 * Protected stuff
	 */
  protected:
	/**
	 * Fields
	 */
	/**
	 * family of this network communication, see socket(3)
	 * Families  are defined in <sys/socket.h>:
	 * PF_UNIX, PF_LOCAL   Local communication  
	 * PF_INET             IPv4 Internet protocols         
	 * PF_INET6           IPv6 Internet protocols
	 * PF_PACKET        Low level packet interface       packet(7)
	 * there are more but these are the moast important 
	 */
	int family;
	/**
	 * Type of the socket also see socket(3)
	 * SOCK_RAW and SOCK_STREAM are the most important
	 * 
	 * SOCK_STREAM
	 *               Provides sequenced, reliable, two-way, connection-based byte  streams.
	 *               An out-of-band data transmission mechanism may be supported.
	 * 
	 * SOCK_SEQPACKET
	 *               Provides a sequenced, reliable, two-way connection-based  data  trans-
	 *               mission  path  for  datagrams  of  fixed maximum length; a consumer is
	 *               required to read an entire packet with each read system call.
	 * SOCK_RAW
	 *               Provides raw network protocol access.
	 * 
	 * 
	 */
	int socktype;
	/**
	 * Protocol to use. see socket(3) and protocols(5)
	 * Just use 0 if you have socktype SOCK_STREAM 
	 */
	int protocol;
	/**
	 * internal handle for this socket
	 */
	int sock;
	/**
	 * last error occured in any socket operation
	 */
	SocketException* lastError;
	/**
	 * time in microsecondss this socket will wait for connection
	 */
	int timeout;
	/**
	 * STL map that shows any refering objects, so we make sure not to destroy that socket while its still referenced
	 * (I got this Idea from www.codefighter.com)
	 */
	map<int,int> references;
	/**
	 * TCP port for network traffic
	 */
	int port;
	/**
	 * C struct to store the relevant information
	 */
	sockaddr_in address;

	/**
	 * Operations
	 */
	/**
	 * decrement the map that indicates all other objects that still have a referenceto this socket
	 * @return 1 if sucessful 0 otherwise
	 */
	int  decRef ();

	
	/**
	 * Increment the map by one so we make sure no reference for this socket exists anywhere in the programm when we destry it.
	 * @return 1 if sucessful 0 otherwise
	 */
	int  incRef ();


	
  };

#endif //SOCKET_H

